//---------------------------------------------------------------------------------------
// This file is part of the Lomse library.
// Lomse is copyrighted work (c) 2010-2016. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//    * Redistributions of source code must retain the above copyright notice, this
//      list of conditions and the following disclaimer.
//
//    * Redistributions in binary form must reproduce the above copyright notice, this
//      list of conditions and the following disclaimer in the documentation and/or
//      other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
// SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// For any comment, suggestion or feature request, please contact the manager of
// the project at cecilios@users.sourceforge.net
//---------------------------------------------------------------------------------------

#include "lomse_time.h"

#include <cmath>

namespace lomse
{

//---------------------------------------------------------------------------------------
//global helper functions to compare times (two floating point numbers)
bool is_equal_time(TimeUnits t1, TimeUnits t2)
{
    return (fabs(t1 - t2) < 0.1);
}

//---------------------------------------------------------------------------------------
bool is_lower_time(TimeUnits t1, TimeUnits t2)
{
    return (t1 < t2) && (fabs(t2 - t1) >= 0.1);
}

//---------------------------------------------------------------------------------------
bool is_greater_time(TimeUnits t1, TimeUnits t2)
{
    return (t1 > t2) && (fabs(t1 - t2) >= 0.1);
}

//---------------------------------------------------------------------------------------
//global function for implementing round-half-up rounding algorithm
TimeUnits round_half_up(TimeUnits num)
{
    return floor(num * 100.0 + 0.5) / 100.0;
}



}   //namespace lomse
